/****************************************
* afficher des carrés dont la couleur rouge est donnée dans un tableau
*
* entree :
*   - x, y : position du coin sup gauche de la ligne de carrés
*   - w : longuer du coté
*   - red : tbaleau qui contient l'intensité du rouge
*
* sortie :
*   - aucune
*****************************************/
void carresRouges(int x, int y, int w, int[] red) {
  for(int i = 0; i < red.length; i++) {
    fill(red[i], 0, 0);
    rect(x + i * w, y, w, w);
  }
}

/****************************************
* afficher des carrés dont la coleur est donnée dans un tableau
*
* entree :
*   - x, y : position du coin sup gauche de la ligne de carrés
*   - w : longuer du coté
*   - red : tbaleau qui contient l'intensité du rouge
*
* sortie :
*   - aucune
*****************************************/
void carres(int x, int y, int w, int[] red, int[] green, int[] blue) {
  for(int i = 0; i < red.length; i++) {
    fill(red[i], green[i], blue[i]);
    rect(x + i * w, y, w, w);
  }
}

/****************************************
* construction d'un tableau avec les n premiers nombres de Fibonacci
*
* entree :
*   - n   : taille du tableau
*
* sortie :
*   - tableau d'entier de taille n 
*****************************************/
int[] creerFibonacci(int n) {
  int[] t = new int[n];
  
  t[0] = 1;
  t[1] = 1;
  
  for(int i = 2; i < n; i++)
    t[i] = t[i-1] + t[i-2];
    
  return t;
}

/****************************************
* augmente toutes les valeurs du tableau
*
* entree :
*   - tab : tableau à modifier
*   - inc : valeur d'incrémentation
*
* sortie :
*   - aucune (tableau modifié lui-même) 
*****************************************/
void addTab(int[] tab, int inc) {
  for(int i = 0; i < tab.length; i++)
    tab[i] += inc;
}

/****************************************
* permutte deux valeurs du tableau
*
* entree :
*   - tab : tableau à modifier
*   - i,j : indice du tableau à modifier
*
* sortie :
*   - aucune (tableau modifié lui-même) 
*****************************************/
void permutte(int[] tab, int i, int j) {
  int tmp = tab[i];
  tab[i]  = tab[j];
  tab[j]  = tmp;
}

/****************************************
* copie le premier tableau dans le second
*
* entree :
*   - source      : tableau origine
*   - destination : tableau cible
*
* sortie :
*   - aucune (cible modifié lui-même) 
*****************************************/
void copie(int[] source, int[] destination) {
  for(int i = 0; i < source.length; i++)
    destination[i] = source[i];
}

/****************************************
* échange les valeurs des 2 tableaux
*
* entree :
*   - tab1 : premier tableau
*   - tab2 : second tableau
*
* sortie :
*   - aucune (tableaux modifiés eux-mêmes) 
*****************************************/
void echange(int[] tab1, int[] tab2) {
  int tmp;
  
  for(int i = 0; i < tab1.length; i++) {
    tmp     = tab1[i];
    tab1[i] = tab2[i];
    tab2[i] = tmp;
  }
}

/****************************************
* permute circulairement le tableau
*
* entree :
*   - tab : tableau origine
*
* sortie :
*   - aucune (tableau modifié lui-même) 
*****************************************/
void permutCirculaire(int [] tab) {
  int tmp = tab[0];
  
  for(int i = 0; i < tab.length - 1; i++)
    tab[i] = tab[i + 1];

  tab[tab.length - 1] = tmp;
}

void setup() {
  size(600, 200);

  int[] fib = creerFibonacci(20);
  carresRouges(0, 0, 10, fib);  

  addTab(fib, 50);
  carresRouges(0, 10, 10, fib);  

  permutte(fib, 1, 18);
  carresRouges(0, 20, 10, fib);  

  int[] cpy = new int[fib.length];
  
  copie(fib, cpy);
  carresRouges(0, 30, 10, cpy);  

  addTab(cpy, 100);  
  carresRouges(0, 50, 10, cpy);  
  carresRouges(0, 60, 10, fib);  

  echange(fib, cpy);
  carresRouges(0, 80, 10, cpy);  
  carresRouges(0, 90, 10, fib);  

  permutCirculaire(fib);
  carresRouges(0, 110, 10, fib);  
}

